        title ""
;;; *********************************************************************
;;; Filename:	main.asm
;;; Purpose:	Main loop for Rotary Display Control
;;; Revision:	1.0
;;; Date:	April 8, 2010
;;; Author:	Patrick Boyd
;;; Copyright:	
;;; *********************************************************************
;;; Revision History:
;;;	
;;; *********************************************************************


;;; *********************************************************************
;;; Assembler Directives, Includes and Defines
;;; *********************************************************************
;{
	list 			; Turn on list output.

	
 		list 			; Turn on list output.

	;; Include register, bit, and other info specific to 
	;; the specified device.	
	ifdef	__16F873
 	#include <p16f873.inc>
	messg "Assembling for PIC16F873."
	endif
	ifdef	__16F874
 	#include <p16f874.inc>
	messg "Assembling for PIC16F874."
	endif
	ifdef	__16F876
 	#include <p16f876.inc>
	messg "Assembling for PIC16F876."
	endif
	ifdef	__16F877
 	#include <p16f877.inc>
	messg "Assembling for PIC16F877."
	endif
	ifdef	__16F873A
	#define A_Device	
 	#include <p16f873a.inc>
	messg "Assembling for PIC16F873A."
	endif
	ifdef	__16F874A
 	#include <p16f874a.inc>
	#define A_Device	
	messg "Assembling for PIC16F874A."
	endif
	ifdef	__16F876A
	#define A_Device	
 	#include <p16f876a.inc>
	messg "Assembling for PIC16F876A."
	endif
	ifdef	__16F877A
	#define A_Device	
 	#include <p16f877a.inc>
	messg "Assembling for PIC16F877A."
	endif


	#define SI1 0
	#define SI2 1
	#define LE	2
	#define OE	3
	#define CLOCK	1
	
	#define SCROLL_DELAY 0x00
	#define MAX_CHARS 0x1C
	#define MAX_COLS 0xE0
	
	#define	MSG1_ADR 0x0A00
	#define MSG2_ADR 0x0B00
	#define MSG3_ADR 0x0C00
	
	errorlevel -302
	
	;}


	

	




	
;;; ---------------------------------------------------------------------
;;; Variable Address Assignments.
;;; ---------------------------------------------------------------------
	;{
	UDATA			; Start of uninitialized data section.
				; Reserve memory for variables using the
				; "RES" directive here.  Note that the
				; linker will assign addresses.
				; eg)  ADcounter  res  1
				;      (reserves 1 byte for ADcounter).



	UDATA_OVR		; Uninitialized data overlay section.
				; Upon occasion, temporary variables
				; can be over-written by other temporary
				; variables.  These would appear here.
				; BUT!  If you ever intend on time-sliced
				; or pre-emptive multitasking		 
				; don't *ever* use this directive!

	UDATA_SHR		; Uninitialized data shared section.
				; Variables that appear here appear
				; across all banks, meaning that no
				; bank selection will be required.
				; Variables of this type are *required*
				; when saving CPU context when an
				; interrupt occurs.  Not all devices
				; have shared memory.  In that case, you
				; need to reserve memory across individual
				; banks at the same offset by using
				; multiple UDATA or UDATA_SHR assignments
				; with an optional address specified.
W_Context:	res 1		; Context saving on interrupt:	W.

	;; For saving context.
STATUS_Context:	res 1		; Saving of STATUS on interrupt.
PCLATH_Context:	res 1		; Saving of PCLATH on interrupt.

	;;	Output to be displayed
OUTPUT_H:	res 1
OUTPUT_L:	res 1

	;; 	Registers for storing temp values for output
TEMP_H:		res 1
TEMP_L:		res 1
COL_DONE:	res 1


ADDR:		res 1

;; Registers for storing message position information

MSG_COUNT:		res 1
OFFSET:			res 1
TMR0_BASE:		res 1
ROTATIONS:		res 1
CUR_COL:		res 1
POSITION:		res 1

;}



;;; ---------------------------------------------------------------------
;;; Establish the OPTION register bit values. 
;;; ---------------------------------------------------------------------
;{

;;; The '__CONFIG' directive is used to embed PIC configuration data 
;;; within an assembly file. The labels following the directive 
;;; are located in the .inc file.  See the device data sheet for
;;; additional information on the configuration word.
	ifndef A_Device	
		__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_ENABLE_ON & _LVP_OFF & _CPD_OFF
	endif
;;; _CP_OFF:	turn off code protection.  Don't change this unless you
;;;             want a device that can never be programmed again.  This
;;;             is a "bug" in some PIC devices.
;;; _WDT_OFF:	turn off the watchdog timer.
;;; _BODEN_ON:	turn on power brown-out reset.
;;; _PWRTE_ON:	turn on power-up timer.
;;; _XT_OSC:	specify that the device is using an XT oscillator.
;;; _WRT_ENABLE_ON:  enable writing to data EEPROM.
;;; _LVP_OFF:	disable low-voltage in-circuit programming.
;;; _CPD_OFF:	disable data EEPROM write protection.

	ifdef A_Device
		messg "A revision device."
	 	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_OFF & _CPD_OFF
	endif

;;; _CP_OFF:	turn off code protection.  Don't change this unless you
;;;             want a device that can never be programmed again.  This
;;;             is a "bug" in some PIC devices.
;;; _WDT_OFF:	turn off the watchdog timer.
;;; _BODEN_ON:	turn on power brown-out reset.
;;; _PWRTE_ON:	turn on power-up timer.
;;; _XT_OSC:	specify that the device is using an XT oscillator.
;;; _WRT_OFF:	disable wrie-prottection of program FLASH.
;;; _LVP_OFF:	disable low-voltage in-circuit programming.
;;; _CPD_OFF:	disable data EEPROM write protection.
	;}
;;; --------------------------------------------------------------------
;;; Main Program.
;;; --------------------------------------------------------------------
;;; --------------------------------------------------------------------
;;; Interrupt Handlers
;;; --------------------------------------------------------------------

ISR:	CODE	0x0004		; Location of interrupt vector.
;{

	;; Save the CPU context.
	;; Note that W_Context was declared under the UDATA_SHR section.
	;; In the PIC16F876/7, it will appear across all banks of memory 
	;; and therefore no bank switching is required. For the
	;; PIC16F873/4, a linker script and use of the UDATA_SHR section
	;; reserves space for W_Context across all banks, even though
	;; there are actually two separate locations.  This code takes
	;; this possible separation into account.  See the linker script
	;; (16F873.lkr), available from the same location as this
	;; template for how this allocation occurs.
	
	movwf	W_Context
	swapf	STATUS,W
	banksel	STATUS_Context	
	movwf	STATUS_Context
	movf	PCLATH,W
	movwf	PCLATH_Context

	;; Why the swapf instruction, above?  That's a good question.
	;; The reason is that, unlike a move instruction, swapf doesn't
	;; affect any bits in STATUS, which is important!
	;; There are situations where you will need to save (and also
	;; restore more than just these two registers.

	;; ------------------------------------------------------------
	;; Your interrupt service routine code goes here.
	;; ------------------------------------------------------------
	;; You'll generally need to follow these steps:
	;; 1.  Determine what caused the interrupt by checking
	;;     appropriate interrupt flags.  You'll usually know which
	;;     ones to check, since you enable interrupts from certain
	;;     peripherals in the main program.
	;; 2.  Take appropriate action.
	;; 3.  Clear the interrupt source.  How to do this depends
	;;     on the peripheral that caused the interrupt, but sometimes
	;;     it is as simple as clearing a flag.
	;; ------------------------------------------------------------

	btfsc	INTCON, TMR0IF	; Check to see if it was the timer that caused the interrupt
	goto	TMR0_ISR
	
;;; ********************************************************************
;;; External Interrupt Service Routine
;;; Description: Finds cycles per column and sets timing accordingly
;;; Triggered By: Hall Sensor
;;; *******************************************************************

EXT_ISR:

	banksel	TMR0
	clrf	CUR_COL
	movf	TMR1H, W		;; Take the MSB of timer1
	sublw	0xFF			;; (ie. timer 1 / 256) subtract it from 
							;; 256 and store it as the base value

	movwf	TMR0_BASE
	movwf	TMR0
	
	;; RPM Test				;; To check RPM of motor uncomment this block
;	movf	TMR1L, W		;; and run BlankLoop
;	movwf	OUTPUT_L
;	movf	TMR1H, W
;	movwf	OUTPUT_H
	;; /RPM Test
	
	clrf	TMR1H
	clrf	TMR1L
	incf	ROTATIONS, F
	bcf		INTCON, INTF
	
	goto	ISR_Restore


;;; ********************************************************************
;;; Timer0 Overflow Interrupt Service Routine
;;; Description: Output this columns display
;;; Triggered By: Timer0 Overflowing indicating a new column needs
;;;			to be drawn
;;; *******************************************************************

TMR0_ISR:					

	call	OUTPUT			; Draw to the LEDs
	banksel	TMR0
	bsf		COL_DONE, 0		; Signal that a column has been drawn
	incf	CUR_COL, F
	movf	TMR0_BASE, W
	movwf	TMR0
	bcf		INTCON, TMR0IF	; Clear the interrupt bit
	goto	ISR_Restore
	


	;; Come back to this point when your interrupt processing is
	;; complete.
ISR_Restore:
	;; Restore the CPU context.
	banksel PCLATH_Context
	movf	PCLATH_Context,W
	movwf	PCLATH
	swapf	STATUS_Context,W
	movwf	STATUS		; Restores whichever bank W_Context
				; actually appears in.
	swapf	W_Context,F	; See the note above as to why swapf
	swapf	W_Context,W	; is used.

	;; Return from the interrupt, which pulls an address off the
	;; (invisible) stack.
	retfie
	;}

	;; ------------------------------------------------------------	
	;; Your main code starts here.  If it just follows the ISR, that
	;; is usually OK.
	;; ------------------------------------------------------------	
	CODE	0x0100

Main:
	;; Program initialization.
;{
	;;;;;
	;;;-----------------------------------------
	; Switch To Bank 4
	;;;-----------------------------------------	
	;;;;;
	banksel EECON1 	; Move to the bank with TRISB
	
	;;;-----------------------------------------
	; Set up EECON1
	;;;-----------------------------------------
	; EEPGD - 1, Use program memory
	; 6:4 - 0, Unimplemented
	; WRERR - 0 Clear Error Bit
	; WREN - 0, Writes not enabled
	; WR - 0, Not writing yet
	; RD - 0, Not reading yet
	;;;-----------------------------------------

	movlw	b'10000000'
	movwf	EECON1
	;;;;;
	;;;-----------------------------------------
	; Switch To Bank 1
	;;;-----------------------------------------	
	;;;;;
	banksel TRISB 	; Move to the bank with TRISB

	;;;-----------------------------------------
	; Set up the TRISA register as follows
	;;;-----------------------------------------
	; RA0 - Output
	; RA1 - Output
	; RA2 - Output
	; RA3 - Output
	; RA4 - Output
	; RA5 - Output
	;;;-----------------------------------------

	clrf TRISA

	;;;-----------------------------------------
	; Set up the TRISB register as follows
	;;;-----------------------------------------
	; RB0 - Input
	; RB1 - Output
	; RB2 - Output
	; RB3 - Output
	; RB4 - Output
	; RB5 - Output
	; RB6 - Output
	; RB7 - Output
	;;;-----------------------------------------

	movlw 	0x01
	movwf 	TRISB

	;;;-----------------------------------------
	; Set up the Options register as follows
	;;;-----------------------------------------
	; RBPU - 1, Pull-ups disabled
	; INTEDG - 0, Trigger on falling edge
	; T0CS	- 0, Use internal clock
	; T0SE	- 0, low-to-high ignored 
	; PSA - 0, PSA assigned to TMR0
	; PS2:PS0 - 000, 1:2 Prescale ratio
	;;;-----------------------------------------
	movlw	b'10000000'
	movwf	OPTION_REG

	;;;;;
	;;;-----------------------------------------
	; Switch To Bank 0
	;;;-----------------------------------------	
	;;;;;

	banksel T1CON

	;;;-----------------------------------------
	; Set up the INTCON register as follows
	;;;-----------------------------------------
	; GIE - 0, Leave Interrupts off for now
	;		   will turn on when initialization done
	; PEIE - 1, Enable Peripheral Interrupts
	; TMR0IE - 1, Enable Timer0 overflow interrupt
	; INTE - 1, Enable RB0/INT External Interrupt
	; RBIE - 0 , Disable RB Port Change Interrupt
	; TMR01F - 0, Clear the Timer0 overflow interrupt flag
	; INTF - 0, Clear the RB0/INT External Interrupt flag
	; RBIF - 0, Clear the RB port Change flag
	;;;-----------------------------------------
	movlw b'01110000'
	movwf INTCON

	;;;-----------------------------------------
	; Set up Timer1
	;;;-----------------------------------------
	; T1CKPS1:0 - 01, 2:1 Prescale Ratio
	; T1OSCEN - 0, Turn Oscillator Off
	; T1SYNC - 0, External Clock sync, ignored
	; TMR1CS - 0, Use Internal Clock 
	; TMR1ON - 0, Turn on at beginning of main loop
	;;;-----------------------------------------
	movlw 	b'00010000'
	movwf	T1CON

	;;;-----------------------------------------
	; Clear Timer1 Registers
	;;;-----------------------------------------	
	clrf	TMR1H
	clrf	TMR1L


	;;;-----------------------------------------
	; Set up PORT variables and clear pins
	;;;-----------------------------------------

	banksel	PORTA 	; Select the bank with PORTA, since we'll be using it

	clrf	PORTA
	clrf	PORTB


	;;;-----------------------------------------
	; Initialize Memory
	;;;-----------------------------------------


	clrf	TMR0_BASE
	
	;;;-----------------------------------------
	; Final Steps
	;;;-----------------------------------------

	banksel	T1CON
	
	bsf		T1CON, TMR1ON	; Start the Timer
	bcf		INTCON, TMR0IF
	bsf		INTCON, GIE		; Enable all interrupts
;}

	goto	Message1Init	;; Jump to whatever main code you want to run
	
;;;	-------------------------------------------------------------------------------------
;;;	For Reference Here are the list of which pins on the PIC
;;;	are associated with which LED driver lines
;;;	RB0 - Hall Sensor
;;;	RA0 - Serial In 1
;;;	RA1 - Serial In 2
;;; RA2 - Latch Enable
;;;	RA3 - Output Enable
;;;	RB1 - Clock
;;; -------------------------------------------------------------------------------------

;;; ********************************************************************
;;; MainLoop
;;; Description: The main process loop
;;; Requires: Nothing!
;;; Returns: I really hope it never returns...
;;; Locations Affected:	Everything...
;;;	Notes: The main loop is responsible for the following things
;;; 		in the following order of importance:
;;;			1) Load new values for the LEDs to display
;;;			2) Scroll the message by updating offset and leading spaces
;;;			3) ???
;;;			4) Profit
;;; ********************************************************************


;;; ********************************************************************
;;; Message 1
;;; ********************************************************************

Message1Init:
	
	banksel	EEADR			
	movlw	0x0A				;; Load the MSBs of the address
	movwf	EEADRH
	clrf	ADDR
	
	;; Initializes the length of the string
	;; Paste code here
	movlw	0xd1				
	movwf	MSG_COUNT
	;; End of pasted code
	clrf	OUTPUT_H
	clrf	OFFSET
	clrf	ROTATIONS
	
Message1:
	movf	OFFSET, W			;; Check to see if the message has completed its rotation
	sublw	0xff
	btfsc	STATUS, Z
	goto	Message2Init

	movf	ROTATIONS, W		;; See if enough rotations have passed to scroll the message
	sublw	SCROLL_DELAY
	btfss	STATUS, C
	goto	Message1Scroll
	goto	Message1Load
	
Message1Scroll:
	incf	OFFSET, F		
	clrf	ROTATIONS
	
Message1Load:
	btfss	COL_DONE, 0			;; If a new message needs to be loaded
	goto	Message1
	bcf		COL_DONE, 0
	
	movf	OFFSET, W			;; EEADR = OFFSET + CUR_COL
	addwf	CUR_COL, W
	
	movwf	EEADR				
	subwf	MSG_COUNT, W		; Check to see if the address is past the end of the string
	btfss	STATUS, C
	goto	Message1Blank
	call	PROGRAM_Read		; Read the data in
	
	banksel	EEDATA
	
	movf	EEDATA, W
	movwf	OUTPUT_L

	goto	Message1
	
Message1Blank:
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	Message1
	
;;; ********************************************************************
;;; Message 2
;;; ********************************************************************

Message2Init:
	
	banksel	EEADR
	movlw	0x0B
	movwf	EEADRH
	clrf	ADDR
	movlw	0xc7
	movwf	MSG_COUNT
	clrf	OUTPUT_H
	clrf	OFFSET
	clrf	ROTATIONS
	
Message2:
	movf	OFFSET, W
	sublw	0xFF
	btfsc	STATUS, Z
	goto	Message3Init

	movf	ROTATIONS, W
	sublw	SCROLL_DELAY
	btfss	STATUS, C
	goto	Message2Scroll
	goto	Message2Load
	
Message2Scroll:
	incf	OFFSET, F
	clrf	ROTATIONS
	
Message2Load:
	btfss	COL_DONE, 0
	goto	Message2
	bcf		COL_DONE, 0
	
	movf	OFFSET, W
	addwf	CUR_COL, W	
	
	movwf	EEADR
	subwf	MSG_COUNT, W
	btfss	STATUS, C
	goto	Message2Blank
	call	PROGRAM_Read
	
	banksel	EEDATA
	
	movf	EEDATA, W
	movwf	OUTPUT_L

	goto	Message2
	
Message2Blank:
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	Message2
	
;;; ********************************************************************
;;; Message 3
;;; ********************************************************************

Message3Init:
	
	banksel	EEADR
	movlw	0x0C
	movwf	EEADRH
	clrf	ADDR
	movlw	0x45
	movwf	MSG_COUNT
	addlw	0x01
	movwf	OFFSET
	clrf	OUTPUT_H
	clrf	ROTATIONS
	
Message3:
	movf	OFFSET, W
	sublw	0xff
	btfsc	STATUS, Z
	goto	BeaverInit

	movf	ROTATIONS, W
	sublw	SCROLL_DELAY
	btfss	STATUS, C
	goto	Message3Scroll
	goto	Message3Load
	
Message3Scroll:
	incf	OFFSET, F
	clrf	ROTATIONS
	
Message3Load:
	btfss	COL_DONE, 0
	goto	Message3
	bcf		COL_DONE, 0
	
	movf	OFFSET, W
	addwf	CUR_COL, W
	
	movwf	EEADR
	subwf	MSG_COUNT, W
	btfss	STATUS, C
	goto	Message3Blank
	call	PROGRAM_Read
	
	banksel	EEDATA
	
	movf	EEDATA, W
	movwf	OUTPUT_L

	goto	Message3
	
Message3Blank:
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	Message3
	
;;; ********************************************************************
;;; Beaver
;;; ********************************************************************
	
BeaverInit:
	
	banksel	EEADR
	movlw	0x0d
	movwf	EEADRH
	clrf	ADDR
	movlw	0x0f
	movwf	MSG_COUNT
	addlw	0x01
	movwf	OFFSET
	clrf	OUTPUT_H
	clrf	OFFSET
	clrf	ROTATIONS
	
Beaver:
	movf	OFFSET, W
	sublw	0xff
	btfsc	STATUS, Z
	goto	Message1Init

	movf	ROTATIONS, W
	sublw	SCROLL_DELAY
	btfss	STATUS, C
	goto	BeaverScroll
	goto	BeaverLoad
	
BeaverScroll:
	incf	OFFSET, F
	clrf	ROTATIONS
	
BeaverLoad:
	btfss	COL_DONE, 0
	goto	Beaver
	bcf		COL_DONE, 0
	
	movf	OFFSET, W
	addwf	CUR_COL, W

	
	movwf	EEADR
	subwf	MSG_COUNT, W
	btfss	STATUS, C
	goto	BeaverBlank
	
	movlw	0x0d
	movwf	EEADRH
	
	call	PROGRAM_Read
	
	banksel	EEDATA
	
	movf	EEDATA, W
	movwf	OUTPUT_H
	
	movlw	0x0e
	movwf	EEADRH
	
	
	call	PROGRAM_Read
	
	banksel	EEDATA
	movf	EEDATA, W
	movwf	OUTPUT_L

	goto	Beaver
	
BeaverBlank:
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	Beaver
	
;;; ********************************************************************
;;; Scrolling Line Test
;;; ********************************************************************
	
	
ScrollInit:
	clrf	ROTATIONS
	clrf	POSITION
	
Scroll:
	movf	POSITION, W
	sublw	0xFF
	btfsc	STATUS, Z
	goto	FillInit

	btfss	COL_DONE, 0
	goto	Scroll
	bcf		COL_DONE, 0
	movf	ROTATIONS, W
	subwf	SCROLL_DELAY, W
	btfss	STATUS, C
	goto	ScrollScroll
	goto	ScrollUpdate
	
ScrollScroll:
	incf	POSITION, F
	clrf	ROTATIONS
	
ScrollUpdate:
	movf	POSITION, W
	subwf	CUR_COL, W
	btfsc	STATUS, Z
	goto	ScrollDraw
	
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	Scroll
	
ScrollDraw:
	movlw	0xFF
	movwf	OUTPUT_H
	movwf	OUTPUT_L
	goto	Scroll
	
;;; ********************************************************************
;;; Scrolling Fill Test
;;; ********************************************************************
	
	
FillInit:
	clrf	ROTATIONS
	clrf	POSITION
	
FillLoop:
	movf	POSITION, W
	sublw	0xFF
	btfsc	STATUS, Z
	goto	EmptyInit

	btfss	COL_DONE, 0
	goto	FillLoop
	bcf		COL_DONE, 0
	movf	ROTATIONS, W
	subwf	SCROLL_DELAY, W
	btfss	STATUS, C
	goto	FillScroll
	goto	FillUpdate
	
FillScroll:
	incf	POSITION, F
	clrf	ROTATIONS
	
FillUpdate:
	movf	POSITION, W
	subwf	CUR_COL, W
	btfss	STATUS, C
	goto	FillDraw
	
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	FillLoop
	
FillDraw:
	movlw	0xFF
	movwf	OUTPUT_H
	movwf	OUTPUT_L
	goto	FillLoop
	
	
;;; ********************************************************************
;;; Scrolling Screen Clear Test
;;; ********************************************************************
	
EmptyInit:
	clrf	ROTATIONS
	clrf	POSITION
	
EmptyLoop:
	movf	POSITION, W
	sublw	0xFF
	btfsc	STATUS, Z
	goto	ScrollInit
	

	btfss	COL_DONE, 0
	goto	EmptyLoop
	bcf		COL_DONE, 0
	movf	ROTATIONS, W
	subwf	SCROLL_DELAY, W
	btfss	STATUS, C
	goto	EmptyScroll
	goto	EmptyUpdate
	
EmptyScroll:
	incf	POSITION, F
	clrf	ROTATIONS
	
EmptyUpdate:
	movf	POSITION, W
	subwf	CUR_COL, W
	btfsc	STATUS, C
	goto	EmptyDraw
	
	clrf	OUTPUT_H
	clrf	OUTPUT_L
	goto	EmptyLoop
	
EmptyDraw:
	movlw	0xFF
	movwf	OUTPUT_H
	movwf	OUTPUT_L
	goto	EmptyLoop
	
	

	
Blank:
	clrf	OUTPUT_H
	clrf	OUTPUT_L	

BlankLoop:
	btfsc	COL_DONE, 0
	bcf		COL_DONE, 0
	goto	BlankLoop
	


	;; Don't let execution fall down to here, otherwise the
	;; subroutines (or whatever follows) will be executed without
	;; parameters, and chaos reigns when a "return" instruction
	;; executes with no return address on the stack!
	
;;; --------------------------------------------------------------------
;;; Application-specific Subroutines.
;;; --------------------------------------------------------------------

;;; ********************************************************************
;;; Subroutine Name: PROGRAM_Read
;;; Description: Reads a value from PROGRAM flash memory
;;; Requires: Address loaded into EADDR and EADDRH
;;; Returns: Nothing
;;; Locations Affected:	Destroys w, changes bank
;;; ********************************************************************

PROGRAM_Read:

;{
	banksel	EECON1
	
	bsf		EECON1, EEPGD
	bsf		EECON1, RD
	
	nop		;; These two ops are ignored
	nop
	
	RETURN
;}


;;; ********************************************************************
;;; Subroutine Name: OUTPUT	
;;; Description: Outputs the values stored in OUTPUT_H and OUTPUT_L to the LEDs
;;; Requires: -The values of OUPUT_H and OUTPUT_L to be set
;;;			  -PORTB bank to be selected
;;; Returns: Nothing
;;; Locations Affected:	W, TEMP_H, TEMP_L
;;; ********************************************************************


OUTPUT:


;{
	
	; Push bits onto LED drivers

	clrw	
	btfsc	OUTPUT_H, 0 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 0	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw	
	btfsc	OUTPUT_H, 1 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 1	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw	
	btfsc	OUTPUT_H, 2 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 2	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw	
	btfsc	OUTPUT_H, 3 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 3	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw
	btfsc	OUTPUT_H, 4 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 4	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw
	btfsc	OUTPUT_H, 5 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 5	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw
	btfsc	OUTPUT_H, 6 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 6	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)

	clrw
	btfsc	OUTPUT_H, 7 ; Test the first bit in OUTPUT_H
	iorlw	0x01		; Set it high if the output bit is 1
	btfsc	OUTPUT_L, 7	; Test the first bit in OUTPUT_L
	iorlw	0x02		; Set it high if the output bit is 1
	movwf	PORTA		; Load it into PORTA
	bsf		PORTB, CLOCK	;	Pulse the Clock (I think the PIC clock is slow enough for this to work
	bcf		PORTB, CLOCK	;	but I might need to add a delay in here)
	
						 
	

	iorlw	0x04	; Re-enable latching
	movwf	PORTA

	RETURN			; Return from subroutine


;}














	
;;; --------------------------------------------------------------------
;;; Code-section Constant Data.
;;; --------------------------------------------------------------------
	ORG	MSG1_ADR

	;;; --------------------------------------------------------------------
	;;; Paste Code For Message 1 Here
	;;; --------------------------------------------------------------------

;{
	DATA	0xff
	DATA	0xff
	DATA	0x3
	DATA	0x3
	DATA	0x3
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x20
	DATA	0x18
	DATA	0x4
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x81
	DATA	0x81
	DATA	0xff
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xfc
	DATA	0xfe
	DATA	0x3
	DATA	0x3
	DATA	0x3
	DATA	0xfe
	DATA	0xfc
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x88
	DATA	0x8c
	DATA	0x8e
	DATA	0xfb
	DATA	0x71
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x22
	DATA	0x73
	DATA	0xd9
	DATA	0x89
	DATA	0x89
	DATA	0xcf
	DATA	0x46
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x81
	DATA	0x81
	DATA	0xff
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xc0
	DATA	0xc0
	DATA	0xff
	DATA	0xff
	DATA	0xc0
	DATA	0xc0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xe0
	DATA	0xf0
	DATA	0x1f
	DATA	0x1f
	DATA	0xf0
	DATA	0xe0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x7e
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x81
	DATA	0xff
	DATA	0x7e
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xd8
	DATA	0xd8
	DATA	0xd8
	DATA	0xd8
	DATA	0xd8
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x98
	DATA	0x98
	DATA	0x98
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x3
	DATA	0x3
	DATA	0x3
	DATA	0x3
	DATA	0x3
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x99
	DATA	0x99
	DATA	0x99
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x88
	DATA	0x8c
	DATA	0x8e
	DATA	0xfb
	DATA	0x71
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xc0
	DATA	0xc0
	DATA	0xff
	DATA	0xff
	DATA	0xc0
	DATA	0xc0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x98
	DATA	0x98
	DATA	0x98
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0










;}
	
	;;; --------------------------------------------------------------------
	;;; End of Pasted Code
	;;; --------------------------------------------------------------------

	ORG	MSG2_ADR
	
	;;; --------------------------------------------------------------------
	;;; Paste Code For Message 2Here
	;;; --------------------------------------------------------------------
	;{
			DATA	0xff
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x81
	DATA	0x81
	DATA	0xc3
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x7e
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x81
	DATA	0xff
	DATA	0x7e
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x20
	DATA	0x10
	DATA	0x20
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x88
	DATA	0x88
	DATA	0x88
	DATA	0xf8
	DATA	0x70
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x3
	DATA	0x3
	DATA	0x3
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xc0
	DATA	0xc0
	DATA	0xff
	DATA	0xff
	DATA	0xc0
	DATA	0xc0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x88
	DATA	0x8c
	DATA	0x8e
	DATA	0xfb
	DATA	0x71
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x20
	DATA	0x18
	DATA	0x4
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x7e
	DATA	0xff
	DATA	0x81
	DATA	0x89
	DATA	0x89
	DATA	0xce
	DATA	0x4f
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x81
	DATA	0x81
	DATA	0xff
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x20
	DATA	0x18
	DATA	0x4
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x88
	DATA	0x8c
	DATA	0x8e
	DATA	0xfb
	DATA	0x71
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x81
	DATA	0x81
	DATA	0xff
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x20
	DATA	0x18
	DATA	0x4
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x7e
	DATA	0xff
	DATA	0x81
	DATA	0x89
	DATA	0x89
	DATA	0xce
	DATA	0x4f
	DATA	0x0
	DATA	0x0
	DATA	0x0



	;}
	;;; --------------------------------------------------------------------
	;;; End of Pasted Code
	;;; --------------------------------------------------------------------


	ORG	MSG3_ADR
	
	;;; --------------------------------------------------------------------
	;;; Paste Code For Message Here
	;;; --------------------------------------------------------------------
	;{
		
		
		DATA	0xff
	DATA	0xff
	DATA	0x81
	DATA	0x81
	DATA	0x81
	DATA	0x81
	DATA	0xc3
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x20
	DATA	0x10
	DATA	0x20
	DATA	0xff
	DATA	0xff
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0x88
	DATA	0x88
	DATA	0x88
	DATA	0xf8
	DATA	0x70
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0xff
	DATA	0xff
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0xdb
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x18
	DATA	0x28
	DATA	0x48
	DATA	0x88
	DATA	0xff
	DATA	0xff
	DATA	0x8
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x72
	DATA	0xf3
	DATA	0x89
	DATA	0x89
	DATA	0x89
	DATA	0xff
	DATA	0x7e
	DATA	0x0
	DATA	0x0
	DATA	0x0
	DATA	0x7e
	DATA	0xff
	DATA	0xa1
	DATA	0x99
	DATA	0x85
	DATA	0xff
	DATA	0x7e
	DATA	0x0
	DATA	0x0
	DATA	0x0



	;}
	;;; --------------------------------------------------------------------
	;;; End of Pasted Code
	;;; --------------------------------------------------------------------
	
	;; This code is for the beaver logo
	
	
	ORG	0x0D00
	
	;;; --------------------------------------------------------------------
	;;; Paste Code Here
	;;; --------------------------------------------------------------------
	;{
		
	DATA 	0x00
	DATA 	0x00
	DATA 	0x01
	DATA 	0x1b
	DATA 	0x1c
	DATA 	0x08
	DATA 	0x15
	DATA 	0x37
	DATA 	0x33
	DATA 	0x10
	DATA 	0x08
	DATA 	0x1c
	DATA 	0x1b
	DATA 	0x01
	DATA 	0x00
	DATA 	0x00

	
	ORG		0x0E00
	DATA 	0x00
	DATA 	0x00
	DATA 	0x80
	DATA 	0xd8
	DATA 	0x3c
	DATA 	0x10
	DATA 	0x48
	DATA 	0x8c
	DATA 	0xcc
	DATA 	0x48
	DATA 	0x50
	DATA 	0x38
	DATA 	0xd8
	DATA 	0x80
	DATA 	0x00
	DATA 	0x00
	
	
	
	



	;}
	;;; --------------------------------------------------------------------
	;;; End of Pasted Code
	;;; --------------------------------------------------------------------


	

;;; ---------------------------------------------------------------------
;;; Vectors Section.
;;; ---------------------------------------------------------------------
;;; Establish the reset vector.  Note that the "0x0000" after the CODE
;;; directive ensures that the linker places the code at program memory
;;; location 0x0000.

Reset:	CODE	0x0000
	pagesel Main		; Selects page of program memory with.
	goto	Main		; the actual program, then goes to it.

	END			; End of file.

